from pyaxidraw import axidraw
from PIL import Image, ImageDraw
from random import choice
from math import *
# Dimensions maximales de la page blanche en pixel
width = 1060
height = 750

# Définission de la marge par rapport à la bordure (Le cadre de bordure est dessiné plus loin dans le programme)
marge = 10
x_min, x_max, y_min, y_max = marge, width - marge, marge, height - marge 

# Définission du padding
padding = 10
x_min, x_max, y_min, y_max = x_min + padding, x_max - padding, y_min + padding, y_max - padding

# Booléen indiquant si le dessin peut être utilisé avec l'AxiDraw. Par défaut, on considérera que oui.

dessin_valide = True

# Initialisation de l'axidraw et enregistrement de son éventuelle connection sous forme d'un booléen
ad = axidraw.AxiDraw()        # Instanciation de l'objet ad de la classe AxiDraw
ad.interactive()              # Utilisation du mode interactif X-Y
axi_connect = ad.connect()    # Connection de l'Axidraw : Renvoie un message d'erreur et False si non connecté

# Configuration des vitesses de l'AxiDraw
if axi_connect:
    ad.options.speed_pendown = 20  # Défini la vitesse lorsque le stylo est baissé
    ad.options.speed_up = 20  # Défini la vitesse lorsque le stylo est baissé
    ad.update() # Met à jour les ré&glages précédents
    ad.penup() # Le stylo est initialement levé
    ad.goto(0, 0) # On positionne le stylo sur les coordonnées initiales

# Initialisation de l'image PIL et de la couleur noire utilisée si l'AxiDraw n'est pas connecté
if not axi_connect:
    image = Image.new(mode ='RGB',size = (width, height), color = (255,255,255)) # Instanciation de l'objet image de la classe Image
    draw = ImageDraw.Draw(image) # Instanciation de l'objet draw de la classe ImageDraw
    black = (0,0,0) # On choisit la couleur noire en RGB

# Dessin de la bordure qui devra contenir l'image 

def bordure():
    points_bordure = [[x_min - padding, y_min - padding] , [x_max + padding, y_min - padding] , [x_max + padding, y_max + padding] , [x_min - padding, y_max + padding] , [x_min - padding, y_min - padding]]
    if axi_connect:
        # Convertion des coordonnées en pouces des coordonnées
        for i in range(len(points_bordure)):
            points_bordure[i][0], points_bordure[i][1] = points_bordure[i][0] * 8.267 / 793.7, points_bordure[i][1] * 8.267 / 793.7
        # On se dirige vers le début de la succession de segments
        ad.goto(points_bordure[0][0], points_bordure[0][1]) # On se dirige vers le début de la succession de segments
        ad.draw_path(points_bordure) # On trace la succession de segments
        ad.penup() # Le stylo est levé
    else:
        for i in range(len(points_bordure)-1): # Tracés des segments
            draw.line((points_bordure[i][0], points_bordure[i][1], points_bordure[i+1][0], points_bordure[i+1][1]), fill = black) 

bordure()

# Fonction vérifiant que les coordonnées se situent bien dans le cadre
def coord_in_cadre(x,y):
    return x_min < x < x_max and y_min < y < y_max

"""
Les fonctions pouvant être utilisées dans le script sont segment(), segments() et polygone(), définies ci-dessous. Il ne faut pas les modifier.
"""

# Fonction traçant un segment en fonction des coordonnées du point de départ et celles du point d'arrivée. Le segment est tracé en levant le stylo avant de se positionner sur le point de départ et le stylo est levé en fin de course.
def segment(x_depart:float, y_depart:float, x_arrivee:float, y_arrivee:float):
    # On vérifie avant toute chose que le segment ne dépassera pas du cadre imposé. Si c'est le cas, il ne sera pas dessiné ,le dessin ne sera pas validé et on sort de la fonction (return)
    if not coord_in_cadre(x_depart,y_depart) or not coord_in_cadre(x_arrivee,y_arrivee):
        # raise ValueError("Dépassement du cadre imposé")
        global dessin_valide
        dessin_valide = False
        print("Segment ne pouvant être tracé")
        return
    if axi_connect and dessin_valide:
        ad.goto(x_depart * 8.267 / 793.7, y_depart * 8.267 / 793.7) # Conversion du déplacement en pouces avec stylo levé (par défaut dans ce programme)
        ad.pendown() # Le stylo est baissé
        ad.lineto(x_arrivee * 8.267 / 793.7, y_arrivee * 8.267 / 793.7) # Conversion du déplacement en pouces avec stylo baissé
        ad.penup()  # Le stylo est relevé en fin de tracé
    else:
        draw.line((x_depart, y_depart, x_arrivee, y_arrivee), fill = black) # Tracé du segment
    
# Fonction traçant une succession de segments en fonction d'une liste d'abscisses et d'une liste d'ordonnées de points successifs. Le stylo est levé avant de se positionner sur le point de départ des segments et se relève à l'arrivée des tracés.
def segments(points:list):
    # Les coordonnées des points successifs sont sous la forme d'une liste de listes. Exemple: points = [[x1,y1],[x2,y2],[x3,y3],[x4,y4]]
    # On vérifie avant toute chose que les segments ne dépasseront pas du cadre imposé. Si c'est le cas, ils ne seront pas dessinés ,le dessin ne sera pas validé et on sort de la fonction (return)
    for i in range(len(points) - 1):
        if not coord_in_cadre(points[i][0],points[i][1]) or not coord_in_cadre(points[i+1][0],points[i+1][1]):
            global dessin_valide
            dessin_valide = False
            print("Segments ne pouvant être tracés")
            return        
    if axi_connect and dessin_valide:
        # Convertion des coordonnées en pouces des coordonnées
        for i in range(len(points)):
            points[i][0], points[i][1] = points[i][0] * 8.267 / 793.7, points[i][1] * 8.267 / 793.7
        # On se dirige vers le début de la succession de segments
        ad.goto(points[0][0], points[0][1]) # On se dirige vers le début de la succession de segments
        ad.draw_path(points) # On trace la succession de segments
        ad.penup() # Le stylo est levé
    else:
        for i in range(len(points)-1): # Tracés des segments
            draw.line((points[i][0], points[i][1], points[i+1][0], points[i+1][1]), fill = black) 

# Fonction traçant une succession de segments en fonction d'une liste d'abscisses et d'une liste d'ordonnées de points successifs. C'est une succession de segments qui se referme automatiquement à la fin, comme un polygône.
def polygone(points:list):
    points.append([points[0][0], points[0][1]])
    segments(points)

def cercle(x_centre, y_centre, rayon):
    return [[rayon*cos(2*pi*t/360)+x_centre,
             rayon*sin(2*pi*t/360)+y_centre]
            for t in range(0,361)]

def rectangle(x_depart,y_depart,delta_x,delta_y):
    polygone([[x_depart,y_depart],[x_depart + delta_x,y_depart],[x_depart + delta_x,y_depart + delta_y],[x_depart,y_depart + delta_y]])

def carre(x_depart,y_depart,cote):
    rectangle(x_depart,y_depart,cote,cote)
    


"""
IMPORTANT
L'abscisse x doit être, à tout moment, comprise entre les valeurs x_min = 20 et x_max = 1040
L'ordonnée y doit être, à tout moment, comprise entre les valeurs y_min = 20 et y_max = 730
"""

# Mon script commence ici

def trait(x_depart: float, y_depart: float, x_arrivee: float, y_arrivee: float, repetitions: int):
    for i in range(repetitions):
        segment(x_depart, y_depart, x_arrivee, y_arrivee)
        x_depart += 4.5
        y_depart -= 2.25
        y_arrivee -= 2.25
        

        

def traits(x_depart: float, y_depart: float, x_arrivee: float, y_arrivee: float, ):
    while x_depart >= 209 and y_depart >= 270:
        segment(x_depart, y_depart, x_arrivee, y_arrivee)
        x_depart -= 1
        x_arrivee += 0.75
        y_depart -= 2.25
        y_arrivee -= 2.25

    while x_depart <= 255 and y_depart >= 273.75:
        segment(x_depart, y_depart, x_arrivee, y_arrivee)
        x_depart += 6.5
        x_arrivee += 0.25
        y_depart -= 2.25
        y_arrivee -= 2.25
        

def traittss(x_depart: float, y_depart: float, x_arrivee: float, y_arrivee: float):
    while x_arrivee <= 360 and y_arrivee >= 290.75:
        segment(x_depart, y_depart, x_arrivee, y_arrivee)
        x_depart += 0.8
        x_arrivee += 1.2
        y_depart -= 2.25
        y_arrivee -= 2.25

    while y_depart >= 278:
        segment(x_depart, y_depart, x_arrivee, y_arrivee)
        x_depart += 0.3
        x_arrivee -= 8
        y_depart -= 2.25
        y_arrivee -= 2.25
        
        
def react(x_depart: float, y_depart: float, x_arrivee: float, y_arrivee: float):
    while  y_arrivee >= 405.15:
        segment(x_depart, y_depart, x_arrivee, y_arrivee)
        x_depart += 0
        x_arrivee += 10
        y_depart -= 5
        y_arrivee -= 5

    while y_arrivee >= 372:
        segment(x_depart, y_depart, x_arrivee, y_arrivee)
        x_depart += 0
        x_arrivee += 0
        y_depart -= 5
        y_arrivee -= 5
        
    while y_arrivee >= 355.875:
        segment(x_depart, y_depart, x_arrivee, y_arrivee)
        x_depart -= 0
        x_arrivee -= 10
        y_depart -= 5
        y_arrivee -= 5
        
        
    
        


 
 



            
        
            
        
        

        
bec = [[30,365],[102,335.8],[102,383.5],[30,365]]
yo = [[102,335.8],[150,317.55],[207,288.35],[225,281.05],[255,273,75],[300,277.4],[360,288.32],[465,319.375],[522,326.675],[544.5,324,85],[579,315.725], [619,299.3],[735,208.05],[828,208.05],[828,193.45],[735,193.45],[735,208.05]]
percheravito = [[153,317.55], [141,295.65],[141,295.65], [117,292], [117,288.35], [147,292],[156,313.9]]
haut_aileron = [[759,193,45],[828,193.45],[804,175.2],[784.5,175.2]]
bas = [[102,385.5],[291,397.85]]
perche = [[292.5,354.05],[291,408.8],[315,405.15],[300,401.5],[303,368,65]]
bass = [[315,406.975],[462,416.1],[546,412.45],[570,412.45],[750,401.5]]
aileron = [[307.5,343.1], [363,348.575], [393,332.15], [369,332.15]]
reacteur = [[750,416.1], [750, 355.875],[780,368.65],[780,405.15]]
trim = [[785,208.05 ], [720,334], [765,334], [814.15,208.05]]
truc = [[742.5,334], [774,334], [774,355.875],[742.5,355.875],[742.5,334],[786,334],[786,355.875],[742.5,355.875]]
cokpit = [[134, 324.85], [210,324.85], [210, 288.35] ,[222, 317.5], [276,317.5], [288,275.575], [294,277.4], [282, 317.5], [345,317.5],[360,290.175]]
aile = [[570,390.55],[465, 372.3], [495, 365], [555,357.7],[615,354.05],[705,365],[654,390.55]]
tetemissile = [[562, 393], [570,390],[570,396]]
queumissile =[[646.5,390],[661.5,393],[646.5,396]]
R = [[285,547.5],[285,456.25], [330,456.25],[330,492.75],[285,492.75],[330,547.5]]
RA = [[345,547.5] , [375,456.25],[405,547.5]]
RAF = [[420,547.5], [420,456.25], [465,456.25]]
RAFA = [[465,547.5] , [495,456.25],[525,547.5]]
RAFAL = [[540,456.25], [540,547.5], [570,547.5]]
RAFALE = [[615,456.25], [585,456.25],[585,547.5],[615,547.5]]
segments(bec)
segments(yo)
segments(percheravito)
polygone(haut_aileron)
segments(bas)
segments(bass)
segments(perche)
polygone(aileron)
polygone(reacteur)
polygone(trim)
polygone(truc)
segments(cokpit)
segments(cercle(650,320,25))
segments(cercle(650,320,12.5))
segments(aile)
rectangle(570,390,45,6)
polygone(tetemissile)
segments(queumissile)
segment(646.5,393,661.5,393)
rectangle(615,390,9,6)
rectangle(624,390,22.5,6)
trait(139, 322, 210,322,17)
traits(222, 317.5, 276 , 317.5)
traittss(282, 317.5, 345,317.5)
react(750,416.1,750,416.1)
segment(570,393,615,393)
segments(R)
segments(RA)
segment(360,501.875,390,501.875)
segments(RAF)
segment(420,492.75,450,492.75)
segments(RAFA)
segment(480,501.875,510,501.875)
segments(RAFAL)
segments(RAFALE)
segment(585,501.875,615,501.875)

# Mon script s'arrête ici

# Sauvegarde de l'image et affichage de l'image
if not axi_connect:
    if not dessin_valide:
        print("Le dessin dépasse du cadre imposé : DESSIN NON-VALIDE\nAppuyez sur Entrer pour continuer")
    else:
        print("Le dessin ne dépasse pas du cadre imposé : DESSIN VALIDE\nAppuyez sur Entrer")
    input()
    nom_dessin = "rafale "
    alphabet = []
    for i in range(65,91):
        alphabet.append(chr(i))
    for i in range(97,123):
        alphabet.append(chr(i))
    for k in range(8):
        nom_dessin += choice(alphabet)
    print(nom_dessin)
    image.save(nom_dessin + ".png", "png")
    image.show()
    
# Deconnection éventuelle de l'Axidraw
if axi_connect:
    ad.disconnect()